from setuptools import setup

setup(
    name='sonetel',
    version='0.2.1',
    packages=['sonetel'],
    url='https://github.com/Sonetel/sonetel-python',
    license='MIT',
    author='aashish',
    author_email='dev.support@sonetel.com',
    description='A simple python wrapper for using Sonetel\'s REST APIs'
)
